"""
Text splitting utilities for RAG ingestion.
"""

from litellm.rag.text_splitters.recursive_character_text_splitter import (
    RecursiveCharacterTextSplitter,
)

__all__ = ["RecursiveCharacterTextSplitter"]

