from typing import Optional

from litellm.types.integrations.custom_logger import StandardCustomLoggerInitParams


class AzureSentinelInitParams(StandardCustomLoggerInitParams):
    """
    Params for initializing an Azure Sentinel logger on litellm
    """

    pass

