"""Google AI Studio Interactions API implementation."""

from litellm.llms.gemini.interactions.transformation import (
    GoogleAIStudioInteractionsConfig,
)

__all__ = ["GoogleAIStudioInteractionsConfig"]
