"""
Pass-Through Endpoint Guardrail Translation

This module exists here (under litellm/llms/) so it can be auto-discovered by
load_guardrail_translation_mappings() which scans for guardrail_translation
directories under litellm/llms/.

The main passthrough endpoint implementation is in:
    litellm/proxy/pass_through_endpoints/

See guardrail_translation/README.md for more details.
"""
