"""
MiniMax LLM Provider
"""

from .text_to_speech.transformation import (
    MinimaxException,
    MinimaxTextToSpeechConfig,
)

__all__ = [
    "MinimaxTextToSpeechConfig",
    "MinimaxException",
]

