"""
DataForSEO Search Module

This module provides search functionality using DataForSEO's SERP API.
DataForSEO offers comprehensive search engine data with high accuracy.
"""

from .transformation import DataForSEOSearchConfig

__all__ = ["DataForSEOSearchConfig"]

